#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.context import makefile, register_ext, type_registry, ErrorString
from pythonite.templating import expand
from pythonite.objects.datetime_obj import DateTime
from pythonite.utils import *
import os, urlparse

class File:
    objtype = 'file'
    def __init__(self, path, context):
        self.path = path
        self.context = context
    def __getattr__(self, attr):
        if attr[0] == '_':
            raise AttributeError, attr
        if attr.startswith('get_'):
            raise AttributeError, attr
        method = getattr(self, 'get_'+attr)
        res = method()
        #setattr(self, attr, res)
        return res
    def readurl(self, url):
        if not url.startswith('http:'):
            url = urlparse.urljoin(self.url, url)
        return URL(url, self.context)
    def get_data(self):
        req = self.context.get('request')
        path = self.path
        dir, name = os.path.split(path)
        path = expand_language(dir, name, req)
        if not os.path.exists(path): # no file? search for a generic skin
            if name == config.index: # index skin
                self.title = os.path.basename(path[:-1-len(config.index)])
                name = os.path.join(config.skindir, config.index)
                path = ''
            else: # media skin
                targetpath = path[:-len(config.extension)]
                if os.path.exists(targetpath):
                    target = makefile(targetpath, self.context)
                    self.context.set(target.objtype, target) # make the target object available for the skin
                    name = os.path.join(config.skindir, target.objtype + config.extension) # skin name
                    path = ''
        try:
            if not path:
                path = path_find(dir, name, req)
            self.data = open(path).read()
        except IOError, why:
            path = path_find(config.skindir, 'error' + config.extension, req)
            error = 'IOError: %s' % why
            self.context.set('error', error)
            req.status = 404
            try:
                self.data = open(path).read()
            except IOError:
                self.data = '<title>Error</title><body>%s</body>' % error
        return self.data
    def get_name(self):
        return os.path.basename(self.path)
    def get_title(self):
	return os.path.splitext(os.path.basename(self.path))[0]
    def get_parents(self):
        root = self.context.get('root')
        path = self.path
        parents = []
        while len(path) > len(root.path):
            path = os.path.dirname(path)
            if path == root.path:
                dir = root
            else:
                dir = makefile(path, self.context)
            parents.insert(0, dir)
        return parents
    def get_csv(self):
        return CSV(self.path, self.context)
    def get_act(self):
        return act(self.path, self.context)
    def get_kbytes(self):
        return (os.path.getsize(self.path)+999)/1000
    def get_url(self):
        curdir = self.context.get('directory').path
	try:
            curdir = os.path.realpath(curdir)
            path = os.path.realpath(self.path)
	except AttributeError:
	    path = self.path # realpath missing under Python2.1
        return relpath(path, curdir)
    def get_date(self):
        try:
            mtime = os.stat(self.path)[8] # .st_mtime
        except OSError:
            path = os.path.dirname(self.path)
            mtime = os.stat(path)[8] # .st_mtime
        return DateTime(mtime)
    def get_next(self):
        return self.skip(1)
    def get_previous(self):
        return self.skip(-1)
    def skip(self, delta):
        dir = makefile(os.path.dirname(self.path), self.context)
        files = dir.list(type=self.objtype)
        for i, file in enumerate(files):
            if file.path == self.path:
                break
        i = (i+delta) % len(files)
        return files[i]
    def __repr__(self):
        return "%s('%s')" % (self.__class__.__name__, self.url)
    def __cmp__(self, other):
        return cmp(self.path, other.path)

register_ext('', File)
