#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.context import register_ext
from pythonite.objects.file_obj import File
import os
import Image as PIL

class Image(File):
    objtype = 'image'
    def get_size(self):
        w, h = PIL.open(self.path).size
        return '%dx%d' % (w, h)
    def get_thumb(self):
        dir, name = os.path.split(self.path)
        dir = os.path.join(dir, '.thumbs')
        if not os.path.exists(dir):
            os.mkdir(dir)
        thumbsize = self.context.get('option').thumbsize
        sizepath = os.path.join(dir, 'SIZE') 
        if not os.path.exists(sizepath) or thumbsize != open(sizepath).read().strip():
            for thumb in os.listdir(dir):
                thumb = os.path.join(dir, thumb)
                os.remove(thumb)
            open(sizepath, 'w').write(thumbsize+'\n')
        path = os.path.join(dir, name) 
        tw, th = map(int, thumbsize.split('x'))
        self.makethumb(self.path, path, (tw, th))
        return Image(path, self.context)
    def makethumb(self, image, thumb, size):
        if not os.path.exists(image):
            return
        if os.path.exists(thumb) and os.path.getmtime(thumb) > os.path.getmtime(image):
            return
        try:
            im = PIL.open(image)
        except:
            return
        x, y = im.size
        if x > size[0]: y = y * size[0] / x; x = size[0]
        if y > size[1]: x = x * size[1] / y; y = size[1]
        size = x, y
        if size != im.size:
            im = im.resize(size, PIL.ANTIALIAS)
        im.save(thumb)

register_ext('.jpg', Image)
register_ext('.png', Image)
register_ext('.gif', Image)
